import { Subject } from 'rxjs';
import { ClientMessageWithoutId, ServerMessageComplete, ServerMessageError, ServerMessageResult } from './contract';
import { StreamBehaviorSubject } from './core';
import { EntityState } from './entity-state';
export declare class MessageSubject<T> extends Subject<T> {
    readonly connectionId: number;
    private onReply?;
    readonly disconnected: Subject<number>;
    protected sendModifier?: (message: ClientMessageWithoutId) => ClientMessageWithoutId;
    constructor(connectionId: number, onReply?: ((message: ClientMessageWithoutId) => MessageSubject<any>) | undefined);
    setSendMessageModifier(modifier: (message: ClientMessageWithoutId) => ClientMessageWithoutId): void;
    /**
     * Sends a message to the server and returns a new MessageSubject.
     * If the connection meanwhile has been reconnected, and completed MessageSubject.
     */
    sendMessage<T = {
        type: '';
    }, K = T | ServerMessageComplete | ServerMessageError>(messageWithoutId: ClientMessageWithoutId): MessageSubject<K>;
    error(err: any): void;
    complete(): void;
    firstOrUndefinedThenClose(): Promise<T | undefined>;
    firstThenClose(): Promise<T>;
}
export declare type PromisifyFn<T extends ((...args: any[]) => any)> = (...args: Parameters<T>) => ReturnType<T> extends Promise<any> ? ReturnType<T> : Promise<ReturnType<T>>;
export declare type RemoteController<T> = {
    [P in keyof T]: T[P] extends (...args: any[]) => any ? PromisifyFn<T[P]> : never;
};
export declare function handleActiveSubject(activeSubject: MessageSubject<ServerMessageResult>, resolve: (v?: any) => void, reject: (error: any) => void, controller: string, name: string, entityState: EntityState, options?: {
    useThisStreamBehaviorSubject?: StreamBehaviorSubject<any>;
}): void;
